import argparse
parser = argparse.ArgumentParser(description =
"""This script takes a MAUVE SNP table and converts it to a TRAMS SNP table.""")
parser.add_argument('-i','--input',type=str,help='Input mauve file.')
parser.add_argument('-o','--output',type=str,help='Name of output file.')
parser.add_argument('-r','--reference',type=str,help='Strain to be used as reference')

Input_name=Output_name=''
InFile_given = OutFile_given = False
args = parser.parse_args()
if args.input:
    Input_name = args.input
    try:
        InFile = open(Input_name,'rU').readlines()
        InFile_given = True
    except IOError as e:
        print "Error trying to open '"+Input_name+"': {1}".format(e.errno, e.strerror)
        print "Please enter an alternative name"
        InFile_given = False

while not InFile_given:
    Input_name = raw_input('Please enter the name of your input file: ')
    try:
        InFile = open(Input_name,'rU').readlines()
        InFile_given = True
    except IOError as e:
        print "Error trying to open '"+Input_name+"': {1}".format(e.errno, e.strerror)
        print "Please enter an alternative name"
        InFile_given = False    

if args.output:
    Output_name = args.output
    try:
        OutFile = open(Output_name,'w')
        OutFile_given = True
    except IOError as e:
        print "Error trying to open '"+Output_name+"': {1}".format(e.errno, e.strerror)
        print "Please enter an alternative name"
        OutFile_given = False

while not OutFile_given:
    Output_name = raw_input('Please enter the name of your output file: ')
    try:
        OutFile = open(Output_name,'w')
        OutFile_given = True
    except IOError as e:
        print "Error trying to open '"+Output_name+"': {1}".format(e.errno, e.strerror)
        print "Please enter an alternative name"
        OutFile_given = False

Ref_name = None
if args.reference:
    Ref_name = args.reference

# First line contains headers, first column contains SNP pattern.
# Ask which strain should be reference, process the rest line by line.
Strains = None
Lines = []

for Line in InFile:

    # First, deal with headers (once)
    if not Strains:
        Headers = Line.strip().split('\t')[1:]
        if Ref_name and Ref_name in Headers:
            Ref_strain = Headers.index(Ref_name)
        else:
            print '\nFound the following strains:'
            for i, Header in enumerate(Headers,1):
                print "{}) {}".format(i, Header)
            Ref_strain = int(raw_input('\nPlease enter the *number* of the desired reference strain: '))-1
        print '\nReference is "'+Headers[Ref_strain]+'"'
        Strains = [Headers.pop(Ref_strain)]+sorted(Headers)
        OutFile.write('\t'.join(['Position']+Strains)+'\n')
        OutFile.flush()
        continue

    # Then, parse lines from input file
    Cells = Line.strip().split('\t')
    SNPs = list(Cells.pop(0))
    if SNPs[Ref_strain] != '-':
        Position = Cells[Ref_strain]
        Lines.append([Position] + [SNPs.pop(Ref_strain)] + SNPs)
##        OutFile.write('\t'.join([Position]+SNPs+['\n']))
##        OutFile.flush()

# Sort lines on position
Lines.sort(key = lambda L: int(L[0]))

# Output lines to output file
OutFile.write('\n'.join(['\t'.join(L) for L in Lines]))

print '\nConversion is done.\n'
OutFile.close()
    
