import vcf
import vcf.filters
import argparse
parser = argparse.ArgumentParser(description =
"""This script takes a VCF file and converts it to a TRAMS SNP table.""")
parser.add_argument('-i','--input',type=str,help='Input VCF file')
parser.add_argument('-o','--output',type=str,help='Name of output file')
parser.add_argument('-t','--threshold',type=float,help='Optional: Quality threshold for in-/exclusion of SNPs')
parser.add_argument('-r','--ref',type=str,help='Name of reference strain')
parser.add_argument('-a','--alt',type=str,help='Name of alt strain')

InFile_given = OutFile_given = False
args = parser.parse_args()
if args.input:
    Input_name = args.input
    try:
        InFile = open(Input_name,'rU')
        InFile_given = True
    except IOError as e:
        print "Error trying to open '"+Input_name+"': {1}".format(e.errno, e.strerror)
        print "Please enter an alternative name"
        InFile_given = False

while not InFile_given:
    Input_name = raw_input('Please enter the name of your input file: ')
    try:
        InFile = open(Input_name,'rU')
        InFile_given = True
    except IOError as e:
        print "Error trying to open '"+Input_name+"': {1}".format(e.errno, e.strerror)
        print "Please enter an alternative name"
        InFile_given = False    

if args.output:
    Output_name = args.output
    try:
        OutFile = open(Output_name,'w')
        OutFile_given = True
    except IOError as e:
        print "Error trying to open '"+Output_name+"': {1}".format(e.errno, e.strerror)
        print "Please enter an alternative name"
        OutFile_given = False

while not OutFile_given:
    Output_name = raw_input('Please enter the name of your output file: ')
    try:
        OutFile = open(Output_name,'w')
        OutFile_given = True
    except IOError as e:
        print "Error trying to open '"+Output_name+"': {1}".format(e.errno, e.strerror)
        print "Please enter an alternative name"
        OutFile_given = False

if args.ref:
    ref_name = args.ref
else:
    ref_name = str(raw_input('Please enter a name for your reference strain: '))
if len(ref_name) == 0:
    ref_name = 'Ref'

if args.alt:
    alt_name = args.alt
else:
    alt_name = str(raw_input('Please enter a name for the alt strain: '))
if len(alt_name) == 0:
    alt_name = 'Alt'

if args.threshold:
    th = args.threshold
else:
    th = raw_input('Please enter a quality threshold value for SNPs (or leave empty for no filter): ')
    try:
        th = float(th)
    except:
        th = None

# Only filter if value for threshold is given
if th:
    Qual = lambda x: x < th
else:
    Qual = lambda x: None

vcf_reader = vcf.Reader(InFile)

OutFile.write('\t'.join(['Pos',ref_name,alt_name])+'\n')

nr = 0

for record in vcf_reader:# Loop through records in vcf file
    # Only process records of type snp, mnp or complex
    if not record.INFO['TYPE'][0].lower() in ['snp','mnp','complex']:
        continue

    # Length of ref and alt needs to be the same, skip otherwise
    if len(record.REF) != len(record.ALT[0]):
        continue

    # Only include records with high quality (filter with set threshold)
    if Qual(record.QUAL): continue
    
    # Split input to separate snps and print to output file
    for i in range(len(record.REF)):
        # Only output lines when Alt != Ref
        if record.REF[i] == str(record.ALT[0])[i]:
            continue
        OutFile.write('\t'.join([str(record.POS+i),record.REF[i],str(record.ALT[0])[i]])+'\n')
        nr += 1
    OutFile.flush()
OutFile.close()

print 'Done, output: '+str(nr)+' SNPs.'
